local ADDON_NAME = ...
local _G = getfenv(0)
local xo, yo = 8, 3 -- X-offset, Y-offset
local equipped = {} -- Table to store equipped items
local db -- Settings on how much we show and where we anchor stuff
local socketsTable = {
	-- /dump string.split(":", GetInventoryItemLink("player", i))
	-- WoD Sockets
	[523] = true, -- Dungeon
	[563] = true, -- Normal Raid
	[564] = true, -- Heroic Raid
	[565] = true, -- Mythic Raid
	-- Prismatic Sockets in 7.0, how many are there?
	[1808] = true, -- From Heroic Dungeons and T19 Normal Raids
	--[3386] = true, -- From Vendor -- Not socket?
	[3458] = true, -- Legendary item with socket?
	--[3459] = true, -- Legendary stuff, but no idea what this does
	-- Are these sockets?
	--[[[3] = true,
	[497] = true,
	[572] = true,]]
	-- Pre 3rd socket
	--[17:04] [2]="|cffe6cc80|Hitem:128827::132832:132811:::::110:258:256:9::119:3:1793:1615:1809:4:664:1736:1601:1809:|h[Xal'atath, Blade of the Black Empire]|h|r"
}
local slotTable = {
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"!Skip!",
	"ChestSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"WristSlot",
	"HandsSlot",
	"Finger0Slot",
	"Finger1Slot",
	"Trinket0Slot",
	"Trinket1Slot",
	"BackSlot",
	"MainHandSlot",
	"SecondaryHandSlot"
}
local f = CreateFrame("Frame", nil, _G.PaperDollFrame) -- iLvel number frame
local g -- iLvel number for Inspect frame
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")

-- Tooltip and scanning by Phanx @ http://www.wowinterface.com/forums/showthread.php?p=271406
local S_ITEM_LEVEL = "^" .. gsub(ITEM_LEVEL, "%%d", "(%%d+)")
local S_UPGRADE_LEVEL = "^" .. gsub(ITEM_UPGRADE_TOOLTIP_FORMAT, "%%d", "(%%d+)")
local S_HEIRLOOM_LEVEL = "^" .. gsub(HEIRLOOM_UPGRADE_TOOLTIP_FORMAT, "%%d", "(%%d+)")

local scantip = CreateFrame("GameTooltip", "iLvlScanningTooltip", nil, "GameTooltipTemplate")
scantip:SetOwner(UIParent, "ANCHOR_NONE")

local function _getRealItemLevel(slotId, unit)
	local realItemLevel, currentUpgradeLevel, maxUpgradeLevel
	local hasItem = scantip:SetInventoryItem(unit, slotId)
	if not hasItem then return nil end -- With this we don't get ilvl for offhand if we equip 2h weapon

	for i = 2, scantip:NumLines() do -- Line 1 is always the name so you can skip it.
		local text = _G["iLvlScanningTooltipTextLeft"..i]:GetText()
		if text and text ~= "" then
			realItemLevel = realItemLevel or strmatch(text, S_ITEM_LEVEL)
			if not (currentUpgradeLevel or maxUpgradeLevel) then
				currentUpgradeLevel, maxUpgradeLevel = strmatch(text, S_UPGRADE_LEVEL)
				if not (currentUpgradeLevel or maxUpgradeLevel) then
					currentUpgradeLevel, maxUpgradeLevel = strmatch(text, S_HEIRLOOM_LEVEL)
				end
			end

			if realItemLevel and currentUpgradeLevel and maxUpgradeLevel then
				return realItemLevel, tonumber(currentUpgradeLevel), tonumber(maxUpgradeLevel)
			end
		end
	end

	return realItemLevel
end

local function _updateItems(unit, frame)
	local tmpItemLevel
	for i = 1, 17 do -- Only check changed player items or items without ilvl text, skip the shirt (4) and always update Inspects
		local itemLink = GetInventoryItemLink(unit, i)
		if i ~= 4 and ((frame == f and (equipped[i] ~= itemLink or frame[i]:GetText() == nil or itemLink == nil and frame[i]:GetText() ~= "")) or frame == g) then
		--if i ~= 4 then
			if frame == f then
				equipped[i] = itemLink
			end

			local realItemLevel, currentUpgradeLevel, maxUpgradeLevel = _getRealItemLevel(i, unit)
			local upgradeString, enchantString, gemString = "", "", ""
			local _, enchantID, upgradeTypeID, numBonuses, affixes

			if frame == g and itemLink and i == 16 then -- When inspecting dualwield Artifact, offhand returns 750
				local _, _, itemRarity = GetItemInfo(itemLink)
				if itemRarity == LE_ITEM_QUALITY_ARTIFACT then
					tmpItemLevel = tonumber(realItemLevel)
				end
			end

			if realItemLevel and db.setting >= 1 then
				if currentUpgradeLevel and maxUpgradeLevel and currentUpgradeLevel < maxUpgradeLevel then
					upgradeString = "|TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:0:0:0:0:32:32:2:30:2:30|t"
				end

				if db.setting == 2 then
					if itemLink then
						_, _, enchantID, _, _, _, _, _, _, _, _, upgradeTypeID, _, numBonuses, affixes = strsplit(":", itemLink, 15)
					end
					enchantID = tonumber(enchantID); numBonuses = tonumber(numBonuses); upgradeTypeID = tonumber(upgradeTypeID);

					if i == 2 or i == 3 or i == 11 or i == 12 or i == 15 then
						-- Neck, Shoulders, Finger0, Finger1, Chest
						if enchantID and enchantID > 0 then
							enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30|t"
						elseif itemLink then
							enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30:221:0:0|t"
						end
					end

					if i == 16 and (upgradeTypeID == 256 or upgradeTypeID == 16777472) then -- Main Hand, Artifact Weapon
						for b = 1, 3 do
							local _, gemLink = GetItemGem(itemLink, b)
							if gemLink and gemLink ~= "" then
								local _, _, _, _, _, _, _, _, _, t = GetItemInfo(gemLink)
								if t and t > 0 then
									gemString = gemString.."|T"..t..":0:0:0:0:32:32:2:30:2:30|t"
								end
							else
								gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Red:0:0:0:0:32:32:2:30:2:30|t"
								equipped[i] = nil -- Might be just missing data from server, try to update next time
							end
						end							
					elseif numBonuses and numBonuses > 0 then
						for b = 1, numBonuses do
							local bonusID = select(b, strsplit(":", affixes))
							if socketsTable[tonumber(bonusID)] then
								local _, gemLink = GetItemGem(itemLink, 1)
								if gemLink and gemLink ~= "" then
									local _, _, _, _, _, _, _, _, _, t = GetItemInfo(gemLink)
									if t and t > 0 then
										gemString = gemString.."|T"..t..":0:0:0:0:32:32:2:30:2:30|t"
									end
								else
									gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Red:0:0:0:0:32:32:2:30:2:30|t"
									equipped[i] = nil -- Might be just missing data from server, try to update next time
								end
							end
						end
					end
				end
			end

			if not realItemLevel or realItemLevel == nil or realItemLevel == "" then
				frame[i]:SetText("")
				equipped[i] = nil -- Might be just missing data from server, try to update next time
			elseif db.setting == 2 then
				if i <= 5 or i == 15 or i == 9 then -- Left side
					if db.inside then
						frame[i]:SetFormattedText("%s%s%s\n%s", enchantString, gemString, upgradeString, realItemLevel)
					else
						frame[i]:SetFormattedText("%s%s%s%s", realItemLevel, enchantString, gemString, upgradeString)
					end
				elseif i == 16 then -- MainHand
					if gemString ~= "" then
						if db.inside then
							frame[i]:SetFormattedText("%s\n%s\n%s", upgradeString, gemString, realItemLevel)
						else
							frame[i]:SetFormattedText("%s\n%s%s", gemString, upgradeString, realItemLevel)
						end
						--[[if frame == f then
							frame[i]:SetWidth(_G.CharacterMainHandSlot:GetWidth() + 2) -- Fix for 3 gems in weapon
						else
							frame[i]:SetWidth(_G.InspectMainHandSlot:GetWidth() + 2) -- Fix for 3 gems in weapon
						end]]
					else
						if db.inside then
							frame[i]:SetFormattedText("%s\n%s", upgradeString, realItemLevel)
						else
							frame[i]:SetFormattedText("%s%s", upgradeString, realItemLevel)
						end
						--[[if frame == f then
							frame[i]:SetWidth(_G.CharacterMainHandSlot:GetWidth())
						else
							frame[i]:SetWidth(_G.InspectMainHandSlot:GetWidth())
						end]]
					end
				elseif i == 17 then -- OffHand
					if tmpItemLevel and tmpItemLevel > 0 then
						frame[i]:SetFormattedText("%s", tmpItemLevel)
					elseif gemString ~= "" then
						if db.inside then
							frame[i]:SetFormattedText("%s\n%s\n%s", upgradeString, gemString, realItemLevel)
						else
							frame[i]:SetFormattedText("%s\n%s%s", gemString, realItemLevel, upgradeString)
						end
					else
						if db.inside then
							frame[i]:SetFormattedText("%s\n%s", upgradeString, realItemLevel)
						else
							frame[i]:SetFormattedText("%s%s", realItemLevel, upgradeString)
						end
					end
				else -- Right side
					if db.inside then
						frame[i]:SetFormattedText("%s%s%s\n%s", upgradeString, gemString, enchantString, realItemLevel)
					else
						frame[i]:SetFormattedText("%s%s%s%s", upgradeString, gemString, enchantString, realItemLevel)
					end
				end
			elseif db.setting == 1 then
				if db.inside then -- Anchor inside
					frame[i]:SetFormattedText("%s\n%s", upgradeString, realItemLevel)
				elseif i <= 5 or i == 15 or i == 9 or i == 17 then -- Left side
					frame[i]:SetText(realItemLevel..upgradeString)
				else
					frame[i]:SetText(upgradeString..realItemLevel)
				end
			else
				frame[i]:SetText(realItemLevel)
			end

			if db.inside or (i == 16 and gemString ~= "") then
				frame[i]:SetWidth(_G.CharacterMainHandSlot:GetWidth() + 2)
			else
				frame[i]:SetWidth(frame[i]:GetStringWidth())
			end
		end
	end
end

local function _returnPoints(number)
	if db.inside == true then
		return "BOTTOM", "BOTTOM", 2, 3
	else
		if number <= 5 or number == 15 or number == 9 then
			return "LEFT", "RIGHT", xo, 0
		elseif number <= 14 then
			return "RIGHT", "LEFT", -xo, 0
		else
			return "BOTTOM", "TOP", 2, yo
		end
	end
end

local function _anchorStrings(frame)
	local point
	if frame == f then
		point = "Character"
	else
		point = "Inspect"
	end

	for i = 1, 17 do
		if i ~= 4 then
			local parent = _G[point..slotTable[i]]
			local myPoint, parentPoint, x, y = _returnPoints(i)
			frame[i]:ClearAllPoints()
			frame[i]:SetPoint(myPoint, parent, parentPoint, x or 0, y or 0)
		end
	end
end

local function _createStrings()
	f:SetFrameLevel(_G.CharacterHeadSlot:GetFrameLevel())

	for i = 1, 17 do
		if i ~= 4 then
			f[i] = f:CreateFontString(nil, "OVERLAY", "GameFontNormalOutline")
		end
	end

	_anchorStrings(f)
	f:Hide()
end

local function _createGStrings()
	g:SetFrameLevel(_G.InspectHeadSlot:GetFrameLevel())

	for i = 1, 17 do
		if i ~= 4 then
			g[i] = g:CreateFontString(nil, "OVERLAY", "GameFontNormalOutline")
		end
	end

	_anchorStrings(g)
	g:Hide()
end

local function OnEvent(self, event, ...) -- Event handler
	if event == "ADDON_LOADED" and (...) == "Blizzard_InspectUI" then
		self:UnregisterEvent(event)

		g = CreateFrame("Frame", nil, _G.InspectPaperDollFrame) -- iLevel number frame for Inspect
		_createGStrings()
		_createGStrings = nil

		_G.InspectPaperDollFrame:HookScript("OnShow", function(self)
			g:SetFrameLevel(_G.InspectHeadSlot:GetFrameLevel())
			f:RegisterEvent("INSPECT_READY")
			_updateItems("target", g)
			g:Show()
		end)

		_G.InspectPaperDollFrame:HookScript("OnHide", function(self)
			f:UnregisterEvent("INSPECT_READY")
			g:Hide()
		end)
	elseif event == "PLAYER_LOGIN" then
		self:UnregisterEvent(event)

		if type(iLevelSetting) == "number" then -- Convert old settings to new
			local previousSetting = iLevelSetting
			iLevelSetting = { setting = previousSetting, inside = false }
		elseif type(iLevelSetting) ~= "table" then -- Create new settings for new users
			iLevelSetting = { setting = 2, inside = false }
		end
		--setting = iLevelSetting
		db = iLevelSetting

		_createStrings()
		_createStrings = nil

		_G.PaperDollFrame:HookScript("OnShow", function(self)
			f:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:RegisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			f:RegisterEvent("ARTIFACT_UPDATE")
			f:RegisterEvent("SOCKET_INFO_UPDATE")
			f:RegisterEvent("COMBAT_RATING_UPDATE")
			_updateItems("player", f)
			f:Show()
		end)

		_G.PaperDollFrame:HookScript("OnHide", function(self)
			f:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:UnregisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			f:UnregisterEvent("ARTIFACT_UPDATE")
			f:UnregisterEvent("SOCKET_INFO_UPDATE")
			f:UnregisterEvent("COMBAT_RATING_UPDATE")
			f:Hide()
		end)
	elseif event == "PLAYER_EQUIPMENT_CHANGED" or event == "ITEM_UPGRADE_MASTER_UPDATE"
	or event == "ARTIFACT_UPDATE" or event == "SOCKET_INFO_UPDATE" or event == "COMBAT_RATING_UPDATE" then
		if (...) == 16 then
			equipped[16] = nil
			equipped[17] = nil
		end
		_updateItems("player", f)
	elseif event == "INSPECT_READY" then
		_updateItems("target", g)
	end
end
f:SetScript("OnEvent", OnEvent)

SLASH_ILEVEL1 = "/ilevel"

SlashCmdList.ILEVEL = function(...)
	if (...) == "0" or (...) == "1" or (...) == "2" or (...) == "inside" then
		if (...) == "inside" then
			db.inside = not db.inside
			_anchorStrings(f)
			if g then
				_anchorStrings(g)
			end
		else
			db.setting = tonumber((...))
		end
		wipe(equipped)
		_updateItems("player", f)
		if g and _G.InspectPaperDollFrame:IsShown() then
			_updateItems("target", g)
		end
	end
	ChatFrame1:AddMessage("|cffff9999"..ADDON_NAME..":|r /ilevel ( 0 | 1 | 2 | inside )\n 0 - Only show item levels.\n 1 - Show item levels and upgrades.\n 2 - Show item levels, upgrades and enchants and gems.\n inside - Change anchor point between INSIDE and OUTSIDE.")
	ChatFrame1:AddMessage("|cffff9999"..ADDON_NAME..":|r Current setting is " .. tostring(db.setting))
	ChatFrame1:AddMessage("|cffff9999"..ADDON_NAME..":|r Current anchor point is " .. (db.inside == true and "INSIDE" or "OUTSIDE"))
end
